// editReserved.js

var reservedError = false;

var reservedLimitToOptions = new Array;

function updateReservedLimitToOptions()
{
  var counter = 0;
  reservedLimitToOptions = new Array;  // empty the array.
  for (var alias in dataObj.zones['external'])
  {
    reservedLimitToOptions[alias] = counter++;
  }
}

function reservedLimitToSelectOptions()
{
  var output = '';
  for (var option in reservedLimitToOptions)
  {
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

// define the <reserved> arguments
nameLookups['reserved'] = new Array;

createLookupEntry('reserved', 'addr', 'Reserved Network', 'mixed', '', 'Specify the network that is not allowed to be originated from in incoming external traffic.  Example: 192.168.0.0/16.<br><br>You can also specify external interfaces to limit this restriction to.');

function displayReservedLimitTo(limitTo)
{
  var output = '';

  for (var aliasIndex in limitTo)
  {
    if (output.length > 0)
    {
      output += ", ";
    }
    output += "<b>" + fixupOutput(limitTo[aliasIndex]) + "</b>";
  }

  return output;
}

function displayReservedEntry(index)
{
  var output = '';
  var entry = dataObj.reserved[index];

  output += '      <td align="right" width="2%"><a href="#" onClick="editReservedEntry(\'' + index + '\'); return false;">' + index + '</a>&nbsp;</td>\n';
  output += '      <td align="left" width="25%">' + fixupOutput(entry.host) + '</td>\n';
  output += '      <td align="center" width="20%">' + displayReservedLimitTo(entry.limitTo) + '</td>\n';
  output += '      <td align="center" width="20%">' + (entry.active ? 'true' : 'false') + '</td>\n';
  output += '      <td align="left" width="25%">' + fixupOutput(entry.comment) + '</td>\n';
  output += '      <td align="center" width="8%"><a href="#" onClick="deleteReservedEntry(\'' + index + '\'); return false;">Delete</a></td>\n';

  return output;
}

function displayReserved()
{
  var content = getContent();
  
  if (! verifyChangesSaved())
  {
    return;
  }
  entryChangesMade = false;

  // make sure we have the latest up-to date array of external aliases.
  updateReservedLimitToOptions();

  displayStatus("Editing Reserved External Networks...<br>\nClick on the Index number of the entry to edit, the <b>Delete</b> link to Delete an entry or the <b>Create</b> button to create a new entry.", "info");

  // build up the form html to display the dataObj.reserved info in.
  var html = "";
  html += '  <center><b>Reserved External Networks</b></center><br>\n';
  html += '  <center><input type="button" name="create" value="Create" onClick="createReservedEntryScreen();"></center>\n';
  html += '  <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += '      <th align="center">Index</th><th align="center">Host</th><th align="center">LimitTo</th><th align="center">Active</th><th align="center">Comment</th><th>&nbsp;</th>\n';
  html += '    </tr>\n';
  var counter = true;
  for (var n=0; n < dataObj.reserved.length; n++)
  {
    if (dataObj.reserved[n] != undefined)
    {
      var color = (counter ? "lightblue" : "white");
      html += '    <tr bgcolor="' + color + '">\n';
      html += displayReservedEntry(n);
      html += '    </tr>\n';
      counter = !counter;
    }
  }
  html += '  </table>\n';
  html += '  <br>\n';
  html += '  <center><input type="button" name="create" value="Create" onClick="createReservedEntryScreen();"></center>\n';

  content.innerHTML = html;
}

function displayReservedEntryScreen(mode, index)
{
  var content = getContent();
  var config = document.configF;
  var reserved = dataObj.reserved;
  var entry;

  if (mode != "Create")
  {
    entry = reserved[index];
  }
  else
  {
    entry = new createReservedEntry("", [], 0, "");
  }

  displayStatus(mode + (mode != "Create" ? "ing" : "") + " Reserved Network Entry" + (mode != "Create" ? ' ' + index : '') + "...", "info");

  // build up the form html to display the dataObj.reserved info in.
  var html = "";
  html += '  <input type="hidden" name="index" value="' + index + '">\n';
  html += '  <center><b>' + mode + ' Reserved Network Entry' + (mode != "Create" ? ' ' + index : '') + '</b></center><br>\n';
  html += '  <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += '      <th align="center">Index</th><th align="center">Host</th><th align="center">LimitTo</th><th align="center">Active</th><th align="center">Comment</th>\n';
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td align="center" width="10%"><a href="#" onClick="displayHelp(\'reserved\', \'addr\'); return false;">' + (mode != "Create" ? index : "&nbsp;") + '</a>&nbsp;</td>\n';
  html += '      <td align="center" width="25%"><input type="text" name="host" value="' + entry.host + '" onChange="entryChangesMade=true;" onKeyPress="entryChangesMade=true;"></td>\n';
  html += '      <td align="center" width="20%">\n';
  html += '        <select name="limitTo" size="5" multiple="multiple" onChange="entryChangesMade=true;">\n';
  html += reservedLimitToSelectOptions();
  html += '        </select>\n<br>\n';
  html += '        ' + selectAllButton('limitTo') + '<br>\n';
  html += '        ' + toggleSelectButton('limitTo') + '\n';
  html += '      </td>\n';
  html += '      <td align="center" width="20%">\n';
  html += '        <select name="active" onChange="entryChangesMade=true;">\n';
  html += trueFalseSelectOptions();
  html += '        </select>\n';
  html += '      </td>\n';
  html += '      <td align="center" width="25%"><input type="text" name="comment" value="' + entry.comment + '" onChange="entryChangesMade=true;" onKeyPress="entryChangesMade=true;"></td>\n';
  html += '    </tr>\n';
  html += '  </table>\n';
  html += '  <br>\n';
  html += '  <center><input type="button" name="update" value="Update" onClick="updateReservedEntry(\'' + mode + '\', \'' + index + '\');"></center>\n';

  content.innerHTML = html;

  // initialize the settings.
  for (var aliasIndex in entry.limitTo)
  {
    var alias = entry.limitTo[aliasIndex];
    config.limitTo.options[reservedLimitToOptions[alias]].selected = true;
  }
  config.active.selectedIndex = boolSelectIndex(entry.active);
}

function editReservedEntry(index)
{
  displayReservedEntryScreen('Edit', index);
}

function deleteReservedEntry(index)
{
  var reserved = dataObj.reserved;
  var result = confirm("Are you sure you want to Delete Reserved Network entry " + reserved[index].host + "?");
  if (result == false)
  {
    return;
  }

  // verify the entry exists.
  if (reserved[index] == null)
  {
    alert("Reserved Network Entry '" + index + "' does not exist to Delete!");
  }
  // delete the entry.
  else
  {
    delete reserved[index];
    changesMade = true;
  }

  // redisplay the Reserved screen.
  displayReserved();
}

function createReservedEntryScreen()
{
  displayReservedEntryScreen('Create', -1);
}

function updateReservedEntry(mode, index)
{
  var config = document.configF;
  var reserved = dataObj.reserved;
  var entry;

  if (mode != "Create")
  {
    entry = reserved[index];
  }
  else
  {
    entry = new createReservedEntry("", [], 0, "");
  }

  if (isTrueSelect(config.active.selectedIndex) != entry.active)
  {
    entryChangesMade = true;
    entry.active = isTrueSelect(config.active.selectedIndex);
  }
  if (config.host.value != entry.host)
  {
    entryChangesMade = true;
    entry.host = config.host.value;
  }
  if (config.comment.value != entry.comment)
  {
    entryChangesMade = true;
    entry.comment = config.comment.value;
  }
  if (config.limitTo.selectedIndex == -1)
  {
    if (entry.limitTo.length > 0)
    {
      // We had entries previously specified and they unselected them all.
      entryChangesMade = true;
      entry.limitTo.length = 0;
    }
  }
  else
  {
    // count up the number of selected entries and compare to the entry.
    var numSelected = 0;
    for (var i=0; i < config.limitTo.options.length; i++)
    {
      if (config.limitTo.options[i].selected)
      {
        numSelected++;
      }
    }
    if (numSelected != entry.limitTo.length)
    {
      entryChangesMade = true;
      entry.limitTo.length = 0;  // make sure it is empty first.
      for (var i=0; i < config.limitTo.options.length; i++)
      {
        if (config.limitTo.options[i].selected)
        {
          entry.limitTo.push(config.limitTo.options[i].value);
        }
      }
    }
  }

  if (entry.host.length == 0)
  {
    alert("You must specify a host value!");
    reservedError = true;
  }

  if (reservedError)
  {
    // reset the error flag.
    reservedError = false;
  }
  else
  {
    if (mode == "Create")
    {
      // figure out the index to insert at (try to fill in any empty holes).
      for (var i=0; i < reserved.length; i++)
      {
        if (reserved[i] == null)
        {
          index = i;
          break;
        }
      }
      if (index == -1)
      {
        index = reserved.length;  // point to the last entry.
      }
    }
    if (entryChangesMade)
    {
      changesMade = true;
      entryChangesMade = false;
      reserved[index] = entry;
    }
    displayReserved();
  }
}
