// editZones.js

var zonesError = false;

var zonesNetworkOptions = new Array;
var zonesBoundToOptions = new Array;

function updateZonesNetworkOptions()
{
  var counter = 0;
  zonesNetworkOptions.length = 0;  // empty the array.
  for (var host in dataObj.networks)
  {
    if (dataObj.networks[host].type == 'normal')
    {
      zonesNetworkOptions[host] = counter++;
    }
  }
}

function updateZonesBoundToOptions()
{
  var counter = 0;
  for (var zoneIndex in zonesBoundToOptions)
  {
    delete zonesBoundToOptions[zoneIndex];  // empty the array.
  }
  var zones = new Array ("external", "internal", "dmz", "dialin");
  for (var zoneIndex in zones)
  {
    var zone = zones[zoneIndex];
    if (zone == 'external' || dataObj.config.zones[zone])
    {
      for (var alias in dataObj.zones[zone])
      {
        if (dataObj.zones[zone][alias].ipsec)
        {
          zonesBoundToOptions[alias] = counter++;
        }
      }
    }
  }
}

function zonesNetworkSelectOptions()
{
  var output = '';
  for (var option in zonesNetworkOptions)
  {
    output += '    <option value="' + option + '">' + dataObj.networks[option].host + (dataObj.networks[option].comment.length > 0 ? ' (' + dataObj.networks[option].comment + ')' : '') +  (!dataObj.networks[option].active ? ' *' : '') + '</option>\n';
  }

  return output;
}

function zonesBoundToSelectOptions()
{
  var output = '';

  for (var option in zonesBoundToOptions)
  {
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

// define the <zones> arguments
nameLookups['zones'] = new Array;

createLookupEntry('zones', 'alias', 'Alias', 'text', '', 'Specify the alias of the interface you are creating.  This is the human readable string to represent eth0 by for example.');
createLookupEntry('zones', 'name', 'Interface Name', 'text', '', 'Specify the name of the interface you are creating.  This is usually eth0, ppp0, br0, tr0, etc.');
createLookupEntry('zones', 'boundTo', 'Bound To', 'select', zonesBoundToSelectOptions, 'Specify the interface (by alias) that this IPSec interface is bound to.');
createLookupEntry('zones', 'ip', 'IP Address', 'text', '', 'Specify the IP Address this interface has associated with it.  If you get your IP via DHCP then use the keyword <b>dynamic</b>.');
createLookupEntry('zones', 'ipsec', 'IPSec Enabled', 'select', trueFalseSelectOptions, 'Do you want IPSec traffic to terminate at this interface?');
createLookupEntry('zones', 'ipsecLimitFrom', 'IPSec Limited From Network(s)', 'multiselect', zonesNetworkSelectOptions, 'Specify the networks/hosts that you want to limit IPSec connections from.  If you select nothing then you will allow IPSec connections from anywhere the interface has in its networks list.  If you are boundTo an external interface then this is essentially the entire world.');
createLookupEntry('zones', 'network', 'Network(s)', 'multiselect', zonesNetworkSelectOptions, 'Specify the networks/hosts that this interface is part of or should allow traffic to flow between.');
createLookupEntry('zones', 'proxyarp', 'Proxy Arp Enabled', 'select', trueFalseSelectOptions, 'Do you want this interface to answer for Proxy Arp requests?');
createLookupEntry('zones', 'active', 'Active', 'select', trueFalseSelectOptions, 'Is this interface active?');
createLookupEntry('zones', 'comment', 'Comment', 'textarea', '', 'Specify any comment about this interface that will help remind you of what it is doing.');

function displayZonesNetworkInfo(network)
{
  var output = '';

  for (var index in network)
  {
    if (dataObj.networks[network[index]] == undefined)
    {
      continue;
    }
    if (output.length > 0)
    {
      output += ", ";
    }
    var comment = dataObj.networks[network[index]].comment;
    output += (comment.length > 0 ? '<span title="' + comment + '">' : '') + "<b>" + fixupOutput(dataObj.networks[network[index]].host) + "</b>" + (comment.length > 0 ? '</span>' : '');
  }

  return output;
}

function displayZoneEntry(zone, alias)
{
  var output = '';
  var entry = dataObj.zones[zone][alias];

  output += '      <td align="center" width="10%"><a href="#" onClick="editZonesEntry(\'' + zone + '\', \'' + alias + '\'); return false;">' + fixupOutput(alias) + '</a>&nbsp;</td>\n';
  output += '      <td align="center" width="10%">' + fixupOutput(entry.name) + '</td>\n';
  if (zone == 'ipsec')
  {
    output += '      <td align="center" width="30%">' + fixupOutput(entry.boundTo) + '</td>\n';
  }
  else
  {
    output += '      <td align="center" width="10%">' + fixupOutput(entry.ip) + '</td>\n';
    output += '      <td align="center" width="10%">' + (entry.ipsec ? 'true' : 'false') + '</td>\n';
    output += '      <td align="center" width="10%">' + displayZonesNetworkInfo(entry.ipsecLimitFrom) + '</td>\n';
  }
  output += '      <td align="left" width="10%">' + displayZonesNetworkInfo(entry.network) + '</td>\n';
  output += '      <td align="center" width="10%">' + (entry.proxyarp ? 'true' : 'false') + '</td>\n';
  output += '      <td align="center" width="10%">' + (entry.active ? 'true' : 'false') + '</td>\n';
  output += '      <td align="left" width="10%">' + fixupOutput(entry.comment) + '</td>\n';
  output += '      <td align="center" width="10%"><a href="#" onClick="deleteZonesEntry(\'' + zone + '\', \'' + alias + '\'); return false;">Delete</a></td>\n';

  return output;
}

function displayZone(zone)
{
  var output = '';
  var interfaces = dataObj.zones[zone];

  output += '  Zone <b>' + fixupOutput(zone) + '</b> &nbsp;&nbsp;<input type="button" name="create' + zone + '" value="Create Interface" onClick="createZonesEntryScreen(\'' + zone + '\');"><br>\n';
  output += '  <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  output += '    <tr>\n';
  output += '      <th align="center">Alias</th><th align="center">Interface</th>';
  if (zone == 'ipsec')
  {
    output += '<th align="center">BoundTo</th>';
  }
  else
  {
    output += '<th align="center">IP Address</th><th align="center">IPSec</th><th align="center">IPSec LimitFrom</th>';
  }
  output += '<th align="center">Network</th><th align="center">Proxy Arp</th><th align="center">Active</th><th align="center">Comment</th><th>&nbsp;</th>\n';
  output += '    </tr>\n';
  var counter = true;
  for (var alias in interfaces)
  {
    var color = (counter ? "lightblue" : "white");
    output += '    <tr bgcolor="' + color + '">\n';
    output += displayZoneEntry(zone, alias);
    output += '    </tr>\n';
    counter = !counter;
  }
  output += '  </table>\n';

  return output;
}

function displayZones()
{
  var content = getContent();

  if (! verifyChangesSaved())
  {
    return;
  }
  entryChangesMade = false;

  // make sure we have the latest up-to date array of networks.
  updateZonesNetworkOptions();
  updateZonesBoundToOptions();

  displayStatus("Editing Zones...<br>\nClick on the Interface name of the entry to edit, the <b>Delete</b> link to Delete an entry or the <b>Create</b> button to create a new entry.", "info");

  // build up the form html to display the dataObj.zones info in.
  var html = "";
  html += '  <center><b>Zones</b></center><br>\n';

  html += displayZone('external');  // This zone is always visible.
  for (var zone in dataObj.config.zones)
  {
    if (dataObj.config.zones[zone])
    {
      html += '  <hr noshade>\n';
      html += displayZone(zone);
    }
  }

  content.innerHTML = html;
}

function displayZoneEntryEditable(name, prefixString, rowSpan)
{
  var output = '';
  try {
    var info = getLookupEntry('zones', name);
  }
  catch (e) {
    return 'Error getting info! ' + e.message;
  }
  var handler = '';
  var result = info.handler.match(/^(.*onChange=".+)(")(.*)$/);
  if (result != null)
  {
    handler = result[1] + '; entryChangesMade=true;"' + result[3];
  }
  else
  {
    handler = 'onChange="entryChangesMade=true;"';
  }

  // create a table to hold the name and user interaction parts in.
  output += '    <td width="50%" valign="top"' + (arguments.length == 3 ? ' rowspan="' + rowSpan + '"' : '') + '>\n';  // start the td we are in.
  if (arguments.length >= 2)
  {
    if (prefixString.length > 0)
    {
      output += '      <center><b>' + prefixString + '</b></center>\n';
    }
  }
  output += '      <table width="100%" border="0" cellpadding="0" cellspacing="2">\n';
  output += '        <tr>\n';
  output += '          <td align="right" width="50%" valign="top"><a href="#" onClick="displayHelp(\'zones\', \'' + name + '\'); return false;">' + info.displayString + '</a>&nbsp;</td>\n';
  output += '          <td align="left" width="50%">';

  if (info.type == "select")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n';
  }
  if (info.type == "text")
  {
    output += '            <input type="text" name="' + name + '" value=""' + handler + ' onKeyPress="entryChangesMade=true;">\n';
  }
  if (info.type == "textarea")
  {
    output += '            <textarea name="' + name + '"' + handler + ' onKeyPress="entryChangesMade=true;"></textarea>\n';
  }
  if (info.type == "multiselect")
  {
    output += '            <select name="' + name + '" multiple="multiple" size="5"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n<br>\n';
    output += '            ' + selectAllButton(name) + '<br>\n';
    output += '            ' + toggleSelectButton(name) + '\n';
  }

  output += '          </td>\n';
  output += '        </tr>\n';
  output += '      </table>\n';
  output += '    </td>\n';  // close the td we are in.

  return output;
}

function displayZonesEntryScreen(mode, zone, alias)
{
  var content = getContent();
  var config = document.configF;
  var zones = dataObj.zones;
  var entry;

  if (mode != "Create")
  {
    entry = zones[zone][alias];
  }
  else
  {
    if (zone == 'ipsec')
    {
      entry = new createIPSecZoneEntry("", "", "", 0, 0, "");
    }
    else
    {
      entry = new createOtherZoneEntry("", "dynamic", 0, "", "", 0, 0, "");
    }
  }

  displayStatus(mode + (mode != "Create" ? "ing" : "") + " Zone '" + fixupOutput(zone) + "' Entry" + (mode != "Create" ? ' ' + alias : '') + "...", "info");

  // build up the form html to display the dataObj.zones info in.
  var html = "";
  html += '  <input type="hidden" name="zone" value="' + zone + '">\n';
  html += '  <input type="hidden" name="origAlias" value="' + alias + '">\n';
  html += '  <center><b>' + mode + ' Zone \'' + fixupOutput(zone) + '\' Entry' + (mode != "Create" ? ' ' + fixupOutput(alias) : '') + '</b><br>\n';
  html += '  <table border="0" width="75%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += displayZoneEntryEditable('alias');
  html += displayZoneEntryEditable('name');
  html += '    </tr>\n';
  if (zone == 'ipsec')
  {
    html += '    <tr>\n';
    html += displayZoneEntryEditable('boundTo');
    html += '      <td>&nbsp;</td>\n';
    html += '    </tr>\n';

    if (numHashEntries(zonesBoundToOptions) == 0)
    {
      alert("You must enable IPSec on some non-IPSec interfaces (zones) before you can create an IPSec interface!");
      displayZones();
      return;
    }
  }
  else
  {
    html += '    <tr>\n';
    html += displayZoneEntryEditable('ip');
    html += '      <td>&nbsp;</td>\n';
    html += '    </tr>\n';
    if (dataObj.config.zones['ipsec'])
    {
      html += '    <tr>\n';
      html += displayZoneEntryEditable('ipsec');
      html += displayZoneEntryEditable('ipsecLimitFrom');
      html += '    </tr>\n';
    }
  }
  html += '    <tr>\n';
  html += displayZoneEntryEditable('network');
  html += displayZoneEntryEditable('proxyarp');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayZoneEntryEditable('active');
  html += displayZoneEntryEditable('comment');
  html += '    </tr>\n';
  html += '  </table>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="update" value="Update" onClick="updateZonesEntry(\'' + mode + '\', \'' + zone + '\', \'' + alias + '\');"></center>\n';

  content.innerHTML = html;

  // initialize the settings.
  config.alias.value = alias;
  config.name.value = entry.name;
  if (zone == 'ipsec')
  {
    config.boundTo.selectedIndex = zonesBoundToOptions[entry.boundTo];
  }
  else
  {
    config.ip.value = entry.ip;
    if (dataObj.config.zones['ipsec'])
    {
      config.ipsec.selectedIndex = boolSelectIndex(entry.ipsec);
      for (var aliasIndex in entry.ipsecLimitFrom)
      {
        var alias = entry.ipsecLimitFrom[aliasIndex];
        config.ipsecLimitFrom.options[zonesNetworkOptions[alias]].selected = true;
      }
    }
  }
  for (var aliasIndex in entry.network)
  {
    var alias = entry.network[aliasIndex];
    config.network.options[zonesNetworkOptions[alias]].selected = true;
  }
  config.proxyarp.selectedIndex = boolSelectIndex(entry.proxyarp);
  config.active.selectedIndex = boolSelectIndex(entry.active);
  config.comment.value = entry.comment;
}

function editZonesEntry(zone, alias)
{
  displayZonesEntryScreen('Edit', zone, alias);
}

function deleteZonesEntry(zone, alias)
{
  var zones = dataObj.zones;
  var configObj = dataObj.config;
  var boundToCount = 0;
  var pathCount = 0;
  var reservedLimitToCount = 0;
  var disableZone = false;
  var result = confirm("Are you sure you want to Delete zone " + zone + "'s interface '" + alias + "'?" + (zone == 'ipsec' ? "\nThis may disable the ipsec flag on the boundTo interface." : ""));
  if (result == false)
  {
    return;
  }

  // verify the entry exists.
  if (zones[zone][alias] == null)
  {
    alert("Zone " + zone + " interface '" + alias + "' does not exist to Delete!");
  }
  // delete the entry.
  else
  {
    // first make sure we won't empty the zone of interfaces
    if (numHashEntries(zones[zone]) == 1)
    {
      result = confirm("Deleting interface '" + alias + "' will empty zone '" + zone + "'!\nPress Ok to delete this interface" + (zone != 'external' ? " and disable the zone" : "") + ", or Cancel to not delete." + (zone == 'ipsec' ? "\n\nThis will disable the ipsec flag on all non-ipsec interfaces." : ""));
      if (result == false)
      {
        return;
      }
      // now we need to flag that the zone needs to be disabled, if not the external zone.
      if (zone != 'external')
      {
        disableZone = true;
      }
    }

    // now make sure we aren't being referenced by any paths or in the case of an ipsec entry, we aren't being boundTo
    boundToCount = findBoundToInterfaces(zone, alias);

    pathCount = findInUsePaths('interface', alias);
    
    reservedLimitToCount = findLimitToReservedEntries(alias);

    var inUseCount = boundToCount + pathCount + reservedLimitToCount;
    var deleteInterface = true;
    if (inUseCount > 0)
    {
      if (zone != 'ipsec' && boundToCount > 0)
      {
        alert("You can not Delete Interface '" + alias + "' as it is boundTo " + boundToCount + " times!\nYou will first need to delete/modify the ipsec interface(s)\nthat are boundTo it before you can delete this interface.");
        deleteInterface = false;
      }
      result = deleteInUsePaths('interface', alias);
      if (result == false)
      {
        return;
      }
      if (reservedLimitToCount > 0)
      {      
        result = deleteReservedLimitToEntry(alias);
        if (result == false)
        {
          return;
        }
      }
    }
    if (deleteInterface)
    {
      if (zone == 'ipsec')
      {
        if (!clearBoundToInterface(zones[zone][alias].boundTo))
        {
          return;
        }
      }
      delete zones[zone][alias];
      delete zones['aliasLookup'][alias];
      if (disableZone)
      {
        if (zone == 'ipsec')
        {
          // make sure all the non-ipsec interfaces don't have the ipsec flag enabled.
          clearAllBoundToInterfaces();
        }
        configObj.zones[zone] = 0;
      }
      changesMade = true;
    }
  }

  // redisplay the Zones screen.
  displayZones();
}

function createZonesEntryScreen(zone)
{
  displayZonesEntryScreen('Create', zone, '');
}

function updateZonesEntry(mode, zone, alias)
{
  var config = document.configF;
  var zones = dataObj.zones;
  var entry;
  var updateAlias = false;

  if (mode != "Create")
  {
    entry = zones[zone][alias];
  }
  else
  {
    if (zone == 'ipsec')
    {
      entry = new createIPSecZoneEntry("", "", [], 0, 0, "");
    }
    else
    {
      entry = new createOtherZoneEntry("", "dynamic", 0, [], [], 0, 0, "");
    }
  }

  if (config.alias.value.length == 0)
  {
    alert("You must specify the alias for the interface to create!");
    config.alias.focus();
    return;
  }

  if (mode != "Create")
  {
    if (config.alias.value != alias)
    {
      // make sure this is not a duplicate alias.
      if (dataObj.zones['aliasLookup'][config.alias.value] != null)
      {
        alert("You already have an interface using alias = '" + config.alias.value + "'!");
        config.alias.focus();
        return;
      }
      else
      {
        alias = config.alias.value;
        updateAlias = true;
      }
    }
  }
  else
  {
    // make sure this is not a duplicate alias.
    if (dataObj.zones['aliasLookup'][config.alias.value] != null)
    {
      alert("You already have an interface using alias = '" + config.alias.value + "'!");
      config.alias.focus();
      return;
    }
    else
    {
      alias = config.alias.value;
    }
  }
  var oldAlias = config.origAlias.value;

  if (config.name.value.length == 0)
  {
    alert("You must specify the interface name!");
    config.name.focus();
    return;
  }

  if (config.name.value != entry.name)
  {
    // make sure this interface name is not already being used.
    for (var tmpZone in dataObj.zones)
    {
      if (tmpZone != 'aliasLookup')
      {
        for (var i in dataObj.zones[tmpZone])
        {
          var info = dataObj.zones[tmpZone][i];
          if (i != oldAlias && info.name == config.name.value)
          {
            alert("You already have an interface using name = '" + config.name.value + "'!");
            config.name.focus();
            return;
          }
        }
      }
    }
    entryChangesMade = true;
    entry.name = config.name.value;
  }

  if (isTrueSelect(config.active.selectedIndex) != entry.active)
  {
    entryChangesMade = true;
    entry.active = isTrueSelect(config.active.selectedIndex);
  }
  if (isTrueSelect(config.proxyarp.selectedIndex) != entry.proxyarp)
  {
    entryChangesMade = true;
    entry.proxyarp = isTrueSelect(config.proxyarp.selectedIndex);
  }
  if (zone == 'ipsec')
  {
    if (config.boundTo.selectedIndex != zonesBoundToOptions[entry.boundTo])
    {
      entryChangesMade = true;
      entry.boundTo = config.boundTo.options[config.boundTo.selectedIndex].value;
    }
  }
  else
  {
    var ipReg = /^(\d+\.\d+\.\d+\.\d+|dynamic)$/;
    if (!ipReg.test(config.ip.value))
    {
      alert("IP = '" + config.ip.value + "' is invalid!");
      zonesError = true;
    }
    if (config.ip.value != entry.ip)
    {
      entryChangesMade = true;
      entry.ip = config.ip.value;
    }
    if (dataObj.config.zones['ipsec'])
    {
      // only check for these values if the ipsec zone is enabled.
      if (isTrueSelect(config.ipsec.selectedIndex) != entry.ipsec)
      {
        if (entry.ipsec)
        {
          // make sure that an ipsec interface isn't boundTo us.
          for (var ipsecAlias in dataObj.zones['ipsec'])
          {
            var info = dataObj.zones['ipsec'][ipsecAlias];
            if (info.boundTo == oldAlias)
            {
              alert("You can not disable IPSec on this interface as it is boundTo by IPSec interface '" + ipsecAlias + "'!");
              config.ipsec.selectedIndex = boolSelectIndex(entry.ipsec);
              zonesError = true;
            }
          }
        }
        entryChangesMade = true;
        entry.ipsec = isTrueSelect(config.ipsec.selectedIndex);
      }
      if (config.ipsecLimitFrom.selectedIndex == -1)
      {
        if (entry.ipsecLimitFrom.length > 0)
        {
          // We had entries previously specified and they unselected them all.
          entryChangesMade = true;
          entry.ipsecLimitFrom.length = 0;
        }
      }
      else
      {
        // count up the number of selected entries and compare to the entry.
        var numDifferent = 0;
        for (var i=0; i < config.ipsecLimitFrom.options.length; i++)
        {
          if (config.ipsecLimitFrom.options[i].selected)
          {
            // see if it was also selected in the entry.
            var found = false;
            for (var j=0; j < entry.ipsecLimitFrom.length; j++)
            {
              if (entry.ipsecLimitFrom[j] == config.ipsecLimitFrom.options[i].value)
              {
                found = true;
              }
            }
            if (!found)
            {
              numDifferent++;
            }
          }
        }
        if (numDifferent > 0)
        {
          entryChangesMade = true;
          entry.ipsecLimitFrom.length = 0;  // make sure it is empty first.
          for (var i=0; i < config.ipsecLimitFrom.options.length; i++)
          {
            if (config.ipsecLimitFrom.options[i].selected)
            {
              entry.ipsecLimitFrom.push(config.ipsecLimitFrom.options[i].value);
            }
          }
        }
      }
    }
  }
  if (config.network.selectedIndex == -1)
  {
    alert("You must select a network that this interface is part of!");
    zonesError = true;
  }
  else
  {
    // Determine if the selection changed.
    var selectionChanged = 0;
    var numSelected = 0;
    for (var i=0; i < config.network.options.length; i++)
    {
      if (config.network.options[i].selected)
      {
        numSelected++;
        var selectionFound = 0;
        for (var j=0; j < entry.network.length; j++)
        {
          if (config.network.options[i].value == entry.network[j])
          {
            selectionFound = 1;
            break;
          }
        }
        if (!selectionFound)
        {
          selectionChanged = 1;
        }
      }
    }
    if (selectionChanged || numSelected != entry.network.length)
    {
      entryChangesMade = true;
      entry.network.length = 0;  // make sure it is empty first.
      for (var i=0; i < config.network.options.length; i++)
      {
        if (config.network.options[i].selected)
        {
          entry.network.push(config.network.options[i].value);
        }
      }
    }
  }
  if (config.comment.value != entry.comment)
  {
    entryChangesMade = true;
    entry.comment = config.comment.value;
  }

  if (zonesError)
  {
    // reset the error flag.
    zonesError = false;
  }
  else
  {
    if (mode == "Create")
    {
      zones['aliasLookup'][alias] = entry.name;
    }
    else if (updateAlias)
    {
      delete zones['aliasLookup'][oldAlias];
      delete zones[zone][oldAlias];
      zones['aliasLookup'][alias] = entry.name;

      // now loop through all ipsec boundTo's and update them as needed.
      if (zone != 'ipsec')
      {
        for (var ipsecAlias in dataObj.zones['ipsec'])
        {
          var info = dataObj.zones['ipsec'][ipsecAlias];
          if (info.boundTo == oldAlias)
          {
            changesMade = true;
            info.boundTo = alias;
            dataObj.zones['ipsec'][ipsecAlias] = info;
          }
        }
      }

      // now update the paths.
      var paths = dataObj.paths;
      for (var serviceIndex in paths['serviceGroupsOrder'])
      {
        var serviceGroup = paths['serviceGroupsOrder'][serviceIndex];
        for (var zoneIndex in paths[serviceGroup]['zonePathOrder'])
        {
          var path = paths[serviceGroup]['zonePathOrder'][zoneIndex];
          for (var infoIndex in paths[serviceGroup][path])
          {
            var info = paths[serviceGroup][path][infoIndex];
            var i = info.i;
            var o = info.o;

            if (i.length > 0)
            {
              for (var iIndex in i)
              {
                if (i[iIndex] == oldAlias)
                {
                  changesMade = true;
                  paths[serviceGroup][path][infoIndex].i[iIndex] = alias;
                  break;
                }
              }
            }
            if (o.length > 0)
            {
              for (var oIndex in o)
              {
                if (o[oIndex] == oldAlias)
                {
                  changesMade = true;
                  paths[serviceGroup][path][infoIndex].o[oIndex] = alias;
                  break;
                }
              }
            }
          }
        }
      }
      
      // now update the Reserved limitTo entries.
      if (findLimitToReservedEntries(oldAlias) > 0)
      {
        updateReservedLimitToEntries(oldAlias, alias);
      }
    }
    if (entryChangesMade || updateAlias)
    {
      changesMade = true;
      entryChangesMade = false;
      zones[zone][alias] = entry;
    }
    displayZones();
  }
}
